/**
 * @file
 * Same Page Preview javascript.
 */

// I don't love that we're using jQuery here, but Drupal's ajax system still
// seems to be closely tied to it.
(function ($, Drupal, once) {
  // Shared application state for same page preview.
  const samePagePreviewState = {
    uuid: null,
    viewMode: 'full',
    previewPaneSrc: null,
    newWindowHref: null,
  }

  /**
   * Update Same Page Preview application state.
   * @param {string} uuid The current node uuid.
   * @param {string} viewMode The view mode to be used for preview.
   */
  const updateSamePagePreviewState = (uuid, viewMode) => {
    const regex = /(\/node\/preview\/)(.*)(\/.*)/;

    // Ensure we have a uuid defined.
    if (uuid) {
      samePagePreviewState.uuid = uuid;
    }
    else if (!samePagePreviewState.uuid) {
      samePagePreviewState.uuid = document.querySelector('iframe.preview').src.match(regex)[2];
    }

    // Update viewMode if it has changed.
    if (viewMode && (samePagePreviewState.viewMode !== viewMode)) {
      samePagePreviewState.viewMode = viewMode;
    }

    // Rather than just refreshing the preview pane, replacing the iframe src
    // using the provided uuid will allow preview to function in cases where the
    // uuid changes.
    if (!samePagePreviewState.previewPaneSrc) {
      samePagePreviewState.previewPaneSrc = document.querySelector('iframe.preview').src;
    }
    samePagePreviewState.previewPaneSrc = samePagePreviewState.previewPaneSrc.replace(regex, `$1${samePagePreviewState.uuid}/${samePagePreviewState.viewMode}?mode=same_page_preview`);

    // The new window button should also be updated to point to the new uuid.
    if (!samePagePreviewState.newWindowHref) {
      samePagePreviewState.newWindowHref = document.querySelector('.button.new-window').href;
    }
    samePagePreviewState.newWindowHref = samePagePreviewState.newWindowHref.replace(regex, `$1${samePagePreviewState.uuid}/${samePagePreviewState.viewMode}`);
  };

  /**
   * Update preview dom elements if the application state has changed.
   * @param {object} previewPane The preview pane iframe.
   * @param {object} newWindowButton The new window button.
   */
  const updateSamePagePreviewDom = (previewPane, newWindowButton) => {
    // Only update form elements if the application state has changed.
    if (samePagePreviewState.newWindowHref !== newWindowButton.href) {
      newWindowButton.href = samePagePreviewState.newWindowHref;
    }
    // The preview pane src always updates so that the iframe is reloaded
    previewPane.src = samePagePreviewState.previewPaneSrc;
  };

  /**
   * Trigger a refresh when the toggle preview button is clicked.
   *
   * @type {Drupal~behavior}
   *
   * @prop {Drupal~behaviorAttach} attach
   *   Attaches the toggle button behavior for.
   */
  Drupal.behaviors.samePagePreviewInit = {
    attach: function (context) {
      // Refresh preview on toggle open.
      once('samePagePreviewInit', '#edit-toggle-preview-link', context).forEach(function (element) {
        element.addEventListener('click', function () {
          $.fn.samePagePreviewTriggerPreview();
        });
      });
    },
  };

  /**
   * Trigger preview.
   */
  $.fn.samePagePreviewTriggerPreview = function() {
    // Could we cut out the middle person in the future? This ajax command just
    // clicks the preview button :)
    const previewButton = document.querySelector('[data-drupal-selector="edit-preview"]');
    previewButton.click();
  };

  /**
   * Re-render the preview pane.
   * @param {string} newUuid The new uuid if the value has changed.
   * @param {string} viewMode The view mode to be used for preview.
   */
  $.fn.samePagePreviewRenderPreview = function(newUuid = null, viewMode) {
    const previewPane = document.querySelector('iframe.preview');
    const newWindowButton = document.querySelector('.button.new-window');

    updateSamePagePreviewState(newUuid, viewMode);
    updateSamePagePreviewDom(previewPane, newWindowButton);
  };
})(jQuery, Drupal, once);

